﻿namespace SelectionModule.Views
{
    using System.Windows;
    using System.Windows.Controls;
    using Infrastructure;
    using Microsoft.Practices.Composite.Events;
    using Microsoft.Practices.Composite.Presentation.Commands;

    /// <summary>
    /// Interaction logic for SelectionView.xaml
    /// </summary>
    public partial class SelectionView : UserControl
    {
        /// <summary>
        /// CAL event aggregator.
        /// </summary>
        private readonly IEventAggregator eventAggregator;

        /// <summary>
        /// Global "Clear" command.
        /// </summary>
        private DelegateCommand<object> clearCommand;

        /// <summary>
        /// The selected radiobutton.
        /// </summary>
        private RadioButton selectedOption;

        /// <summary>
        /// Initializes a new instance of the SelectionView class.
        /// </summary>
        /// <param name="eventAggregator">The injected CAL event aggregator.</param>
        /// <param name="commandProxy">The injected proxy to the global 'Clear' command.</param>
        public SelectionView(
            IEventAggregator eventAggregator,
            ClearCommandProxy commandProxy)
        {
            InitializeComponent();

            this.clearCommand = new DelegateCommand<object>(this.Clear, this.CanClear);

            this.eventAggregator = eventAggregator;
            commandProxy.ClearCommand.RegisterCommand(this.clearCommand);
        }

        /// <summary>
        /// Returns whether or not the selection can be cleared.
        /// </summary>
        /// <param name="o">Command parameter - unused.</param>
        /// <returns>Always true.</returns>
        private bool CanClear(object o)
        {
            return true;
        }

        /// <summary>
        /// Clears the current selection, and notifies.
        /// </summary>
        /// <param name="o">Command parameter - unused.</param>
        private void Clear(object o)
        {
            if (this.selectedOption != null)
            {
                this.selectedOption.IsChecked = false;
            }

            this.eventAggregator.GetEvent<Infrastructure.StatusReportedEvent>().Publish("Selection was cleared.");
        }

        /// <summary>
        /// Current selection changed.
        /// </summary>
        /// <param name="sender">Sender of the event (always the StackPanel).</param>
        /// <param name="e">Event arguments.</param>
        private void RadioButton_Checked(object sender, RoutedEventArgs e)
        {
            this.selectedOption = e.Source as RadioButton;

            string status = string.Format(
                "You selected '{0}'.",
                this.selectedOption.Content.ToString());

            this.eventAggregator.GetEvent<Infrastructure.StatusReportedEvent>().Publish(status);
        }
    }
}
