﻿namespace StatusModule.PresentationModels
{
    using System.ComponentModel;
    using Microsoft.Practices.Composite.Events;
    using Microsoft.Practices.Composite.Presentation.Events;
    using Microsoft.Practices.Unity;

    /// <summary>
    /// PresentationModel for the Status module.
    /// </summary>
    public class StatusPresentationModel : IStatusPresentationModel, INotifyPropertyChanged
    {
        /// <summary>
        /// Corresponding view according the MVP pattern.
        /// </summary>
        private readonly Views.IStatusView view;

        /// <summary>
        /// CAL Unity container.
        /// </summary>
        private readonly IUnityContainer container;

        /// <summary>
        /// CAL event aggregator.
        /// </summary>
        private readonly IEventAggregator eventAggregator;

        /// <summary>
        /// The message to be displayed.
        /// </summary>
        private string statusMessage;

        /// <summary>
        /// Initializes a new instance of the StatusPresentationModel class.
        /// </summary>
        /// <param name="container">The injected CAL Unity container.</param>
        /// <param name="eventAggregator">The injected CAL event aggregator.</param>
        /// <param name="view">The corresponding MVP view.</param>
        public StatusPresentationModel(
            IUnityContainer container,
            IEventAggregator eventAggregator,
            Views.IStatusView view)
        {
            this.container = container;
            this.eventAggregator = eventAggregator;
            this.view = view;
            view.Model = this;

            eventAggregator.GetEvent<Infrastructure.StatusReportedEvent>()
                .Subscribe(this.OnAppStatusChanged, ThreadOption.UIThread, true);

            this.StatusMessage = "The status module is operational.";
        }

        /// <summary>
        /// PropertyChanged notification.
        /// </summary>
        /// <remarks>Member of INotifyPropertyChanged.</remarks>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Gets the associated view, according the MVP pattern.
        /// </summary>
        public Views.IStatusView View
        {
            get
            {
                return this.view;
            }
        }

        /// <summary>
        /// Gets or sets the message to be displayed.
        /// </summary>
        public string StatusMessage
        {
            get
            {
                return this.statusMessage;
            }

            set
            {
                this.statusMessage = value;
                this.NotifyPropertyChanged("StatusMessage");
            }
        }

        /// <summary>
        /// Local event handler for the global event.
        /// </summary>
        /// <param name="message">Statusmessage to be displayed.</param>
        private void OnAppStatusChanged(string message)
        {
            this.StatusMessage = message;
        }

        /// <summary>
        /// PropertyChanged helper.
        /// </summary>
        /// <param name="propertyName">Name of the changed Property.</param>
        private void NotifyPropertyChanged(string propertyName)
        {
            if (this.PropertyChanged != null)
            {
                this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}
